#ifndef BUILD_SQUARE_H
#define BUILD_SQUARE_H

/// build a square using Gmsh API

#include "femGmsh.h"

inline void build_square(double cx = 0.0, double cy = 0.0,
                         double Lx = 1.0, double Ly = 1.0,
                         int nx = 2, int ny = 2,
                         bool structured = true, bool recombine = true)
{
    gmsh::model::add("square");
    double lc = 1.0; // no impact
    gmsh::model::geo::addPoint(cx, cy, 0, lc, 1);
    gmsh::model::geo::addPoint(cx + Lx, cy, 0, lc, 2);
    gmsh::model::geo::addPoint(cx + Lx, cy + Ly, 0, lc, 3);
    gmsh::model::geo::addPoint(cx, cy + Ly, 0, lc, 4);

    gmsh::model::geo::addLine(1, 2, 1);
    gmsh::model::geo::addLine(2, 3, 2);
    gmsh::model::geo::addLine(4, 3, 3);
    gmsh::model::geo::addLine(1, 4, 4);

    gmsh::model::geo::addCurveLoop({1, 2, -3, -4}, 1);

    gmsh::model::geo::addPlaneSurface({1}, 1);

    gmsh::model::geo::synchronize(); // sinon warning "unknown entity..." lors du addPhysicalGroup

    // mesh lines
    gmsh::model::mesh::setTransfiniteCurve(1, nx + 1);
    gmsh::model::mesh::setTransfiniteCurve(3, nx + 1);
    gmsh::model::mesh::setTransfiniteCurve(2, ny + 1);
    gmsh::model::mesh::setTransfiniteCurve(4, ny + 1);

    // recombine
    if (structured)
        gmsh::model::mesh::setTransfiniteSurface(1);
    if (recombine)
        gmsh::model::mesh::setRecombine(2, 1); // quads

    // physical groups - corners
    auto pt1 = gmsh::model::addPhysicalGroup(0, {1});
    auto pt2 = gmsh::model::addPhysicalGroup(0, {2});
    auto pt3 = gmsh::model::addPhysicalGroup(0, {3});
    auto pt4 = gmsh::model::addPhysicalGroup(0, {4});
    gmsh::model::setPhysicalName(0, pt1, "pt1");
    gmsh::model::setPhysicalName(0, pt2, "pt2");
    gmsh::model::setPhysicalName(0, pt3, "pt3");
    gmsh::model::setPhysicalName(0, pt4, "pt4");

    // physical groups - lines
    auto bottom = gmsh::model::addPhysicalGroup(1, {1}, 1000);
    auto right = gmsh::model::addPhysicalGroup(1, {2});
    auto top = gmsh::model::addPhysicalGroup(1, {3});
    auto left = gmsh::model::addPhysicalGroup(1, {4});
    gmsh::model::setPhysicalName(1, bottom, "bottom");
    gmsh::model::setPhysicalName(1, right, "right");
    gmsh::model::setPhysicalName(1, top, "top");
    gmsh::model::setPhysicalName(1, left, "left");

    // physical groups - surfaces
    auto interior = gmsh::model::addPhysicalGroup(2, {1});
    gmsh::model::setPhysicalName(2, interior, "interior");

    // finalize and mesh the geometry
    gmsh::model::mesh::generate(2);

    // gmsh::write("square.msh");
}

#endif //BUILD_SQUARE_H
